/***********************************************************************/
/* FreeVR OpenGL tutorial example 0 w/ GLUT: ex0_glut.c                */
/* Last tested with: N/A (since this is not a VR program)              */
/* Last Modified: December 7, 2009                                     */
/* URL: http://freevr.org                                              */
/***********************************************************************/
/*                                                                     */
/* This example code demonstrates an OpenGL GLUT application program   */
/*   that displays a blue cube and a red pyramid in a simple X window. */
/*                                                                     */
/*                                                                     */
/* The features of this example include:                               */
/*   - Uses GLUT to open an X window                                   */
/*   - Creates an OpenGL viewport                                      */
/*   - Renders a simple static world                                   */
/*                                                                     */
/*                                                                     */
/* Considerations:                                                     */
/*   - Less code is necessary to open the graphics window              */
/*   - The main loop would typically watch for many more input events  */
/*   - This is not a VR program                                        */
/*                                                                     */
/* This application uses the files:                                    */
/*   - shapes.c  (Bill Sherman's simple OpenGL shapes)                 */
/*                                                                     */
/* Copyright 2010, William R. Sherman, All Rights Reserved.            */
/*   In particular, the right to republish this file in whole or in    */
/*   part, in any form (including electronic forms) is prohibited      */
/*   without written consent of the copyright holder.                  */
/***********************************************************************/
#include <stdio.h>	/* needed for printf(), etc. */
#include <stdlib.h>	/* needed for exit() */

#include <GL/glut.h>	/* TODO: CQ3A uses "GLUT/glut.h" when MACOSX is defined */

void	exit(int status);
void	draw_world(void);			/* graphically render the world    */
void	glut_display_func(void);

#define WINDOW_WIDTH	300
#define WINDOW_HEIGHT	300

/***********************************************************************/
main(int argc, char* argv[])
{
	/************************************/
	/*** initialize GL-GLUT routines  ***/
	/************************************/
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);
	glutInitWindowPosition(50, 50);
	glutCreateWindow("simple-glut");

	/*** configure the OpenGL context for rendering ***/
	glEnable(GL_DEPTH_TEST);/* enable depth buffering */
	glDepthFunc(GL_LESS);	/* pedantic, GL_LESS is the default */
	glClearDepth(1.0);	/* pedantic, 1.0 is the default */
	/* frame buffer clears should be to black */
	glClearColor(0.4, 0.0, 0.0, 0.0);
	/* set up projection transform */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-1.0, 1.0, -1.0, 1.0, 1.0, 10.0);
	/* establish initial viewport */
	glViewport(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT);	/* pedantic, full window size is default viewport */

	glutDisplayFunc(glut_display_func);
	glutIdleFunc(glut_display_func);

	/***************************/
	/*** do world simulation ***/
	/***************************/
	glutMainLoop();

	/*********************/
	/*** close up shop ***/
	/*********************/
	exit(0);
}



/* ----------------8<-----------------8<-----------------8<----------------*/
/* In a non-example applicatation, the following would be a separate file. */

		/**************************/
		/**************************/
		/** The Graphics section **/
		/**************************/
		/**************************/

/* functions from shapes.c */
void	draw_cube();
void	draw_pyramid();


/*************************************************************/
/* glut_display_func(): handles the pre and post rendering   */
/*   functions surrounding a call to the world rendering     */
/*   routine.                                                */
void glut_display_func()
{
	/*** Handle projection ***/
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0, -3.0, 0.0);

	/*** Render the world ***/
	draw_world();

	/*** Swap buffers ***/
	glutSwapBuffers();
}


/*************************************************************/
/* draw_world(): Render the two objects in the world -- the  */
/*   blue cube, and red pyramid -- such that they are on the */
/*   XZ plane (typically the floor), and 6 feet forward      */
/*   (typically on the other side of a CAVE's front screen). */
/*************************************************************/
void draw_world(void)
{
	/**************************************/
	/* set the polygon shading parameters */
	glShadeModel(GL_SMOOTH);
	glLineWidth(2.5);

	/**************************************/
	/* clear the screen -- very important */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glClearDepth(1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/************************/
	/* draw all the objects */

	/* a blue cube */
	glColor3ub(100, 100, 255);
	glTranslatef(-2.0, 1.0, -6.0);
	draw_cube();
	glColor3ub(0, 0, 0);
	draw_cube_outline();

	/* a red pyramid */
	glColor3ub(255, 100, 100);
	glTranslatef(4.0, 0.0, 0.0);
	draw_pyramid();
	glColor3ub(0, 0, 0);
	draw_pyramid_outline();
}

